/**
 * Diese Klasse baut die elementaren Aktionen eines Spielers zu Strings zusammen,
 * die der rcssserver interpretiert und reicht diese Strings zum Verschicken an
 * <code>ServerCommunication</code> weiter.
 *
 * Sie orientiert sich voll an der Klasse Actor des SimpleClient,
 * erweitert diese aber.
 *
 * @author Gruppe 11 "DieElf"
 * @author <br>Ansprechpartner: <a href="mailto:brunnerrene@gmx.de">Ren Brunner</a>
 * @author <br>Ansprechpartner: <a href="mailto:fornarina@web.de">Diana Wachsmann</a>
 * @version $Id: Actions.java,v 1.4 2002/07/07 20:56:55 mf Exp $
 */
 
public class Actions
{
    /** Referenz zu ServerCommunication */
    private ServerCommunication serverCommunication;

    /**
     * Konstruktor fr <code>Actor</code>
     *
     * @param serverCommunication vom Typ <code>ServerCommunication</code> ist
     * eine Referenz auf die Schnittstelle zum rcssserver
     * @see ServerCommunication
     */
    public Actions(ServerCommunication serverCommunication)
    {
        this.serverCommunication = serverCommunication;
    }

    /**
     * Mit dieser Methode werden die zehn Spieler automatisch angemeldet, so dass
     * Eingabe per Hand wegfllt.
     * Voraussetzung: rccsserver version 7.0
     *
     * @param teamName vom Typ String 
     */
    public void init(String teamName)
    {
        serverCommunication.send(new String("(init " + teamName + "(version 7.0))"));
    }
    
    
    /**
     * Mit dieser Methode wird der Thorvart automatisch angemeldet, so dass
     * Eingabe per Hand wegfllt.
     * Voraussetzung: rccsserver version 7.0
     *
     * @param teamName vom Typ String 
     */
    public void initKahn(String teamName)
    {
        serverCommunication.send(new String("(init " + teamName + "(version 7.0)(goalie))"));
    }
    
    
    /**
     * Bewegt den Spieler.
     *
     * @param power vom Typ <code>double</code>, beinhaltet die Strke der Beschleunigung.
     */
    public void dash(double power)
    {
        serverCommunication.send(new String("(dash " + power + ")"));
    }

    /**
     * Schiet mit bergebener Strke und in bergebene Richtung.
     *
     * @param power vom Typ <code>double</code>, bestimmt die Strke des Kicks
     * @param direction vom Typ <code>double</code>, bestimmt die Richtung des Kicks
     */
    public void kick(double power, double direction)
    {
        serverCommunication.send(new String("(kick " + power + " " + direction + ")"));
    }

    /**
     * Positioniert den Spieler auf bergebene x,y-Koordinaten.
     * Verwendet das Koordinatensystem des rcssservers.
     * Nur in folgenden Situationen mglich:
     * <ul>
     * <li>vor dem Anspiel</li>
     * <li>nach einem Tor</li>
     * </ul>
     *
     * @param x vom Typ <code>double</code>, bestimmt die x-Koordinate der Position <br>
     * Werte aus [-52.5,52.5]
     * @param y vom Typ <code>double</code>, bestimmt die y-Koordinate der Position <br>
     * Werte aus [-34.0,34.0]
     */
    public void move(double x, double y)
    {
        serverCommunication.send(new String("(move " + x + " " + y + ")"));
    }

    /**
     * Dreht den Krper des Spielers.
     *
     * @param moment ist der Winkel, um den im Uhrzeigersinn gedreht wird.
     * Absolutwerte mssen kleiner als 180 sein, Drehungen nach links mssen mit 
     * negativen Vorzeichen gekennzeichnet sein.
     */
    public void turn(double moment)
    {
        serverCommunication.send(new String("(turn " + moment + ")"));
    }


    /**
     * Dreht den Kopf des Spielers
     *
     * @param neckMoment vom Typ <code>double</code> ist der Winkel um
     * den sich der Kopf dreht
     */
	public void turnNeck(double neckMoment)
    {
        serverCommunication.send(new String("(turn_neck " + neckMoment + ")"));
    }
   
    /**
     * Der Spieler fngt den Ball.
     *
     * @param side vom Typ <code>char</code>, bestimmt auf welcher Seite der Ball
     * angenommen wird.
     */ 
    public void hold(char side)
    {
        serverCommunication.send(new String("(catch " + side + ")"));
    }
    
    /**
     * Meldet den Spieler beim Server ab.
     */
	public void bye()
	{
		serverCommunication.send(new String("(bye)"));
	}
}