import java.util.*;
/**
 * Diese Klasse dient zum Speichern vergangener Informationen
 *
 * @author Gruppe 11 "DieElf"
 * @author <br>Ansprechpartner: <a href="mailto:rene@eDynamic.de">Ren Kubach</a>
 * @author <br>Ansprechpartner: <a href="mailto:fooxbox@web.de">Markus Fuchs</a>
 * @version $Id: Memory.java,v 1.9 2002/07/07 20:56:55 mf Exp $
 */


public class Memory
{
	/** Datenkontainer fr Informationen ber den Ball */
	private Vector ballInfoList = new Vector();
	/** Maximale Anzahl der gespeicherten Informationen ber den Ball */
	private final int ballInfoAmount = 10;

	/** Datenkontainer fr Informationen ber den Spieler */
	private Vector myselfInfoList = new Vector();
	/** Maximale Anzahl der gespeicherten Informationen ber den Spieler */
	private final int myselfInfoAmount = 5;	
	
	/** Datenkontainer fr gemerkte Aktionen des Spielers */
	private Vector actionBufferList = new Vector();
	
	
	/**
	 * Datenkontainer fr prognostizierte Informationen
	 * ber den Spieler
	 */
	private Vector myselfPredictionList = new Vector();
	/** Maximale Anzahl der gespeicherten Informationen ber den Spieler */
	private final int myselfPredictionAmount = 5;	

	/**
	 * Konstruktor
	 */
	public Memory() {
	}
	
/*----------------------------- interne Methoden ----------------------------*/

	/**
	 * Methode zum Einfgen einer Information in eine begrenzte Liste.<br>
	 * Durch die Begrenzung und das zeitlich sortierte Einfgen wird die Liste
	 * zu einer Warteschlage nach dem FIFO-Prinzip
	 *
	 * @see ObjectInfo
	 * @param or, eine <code>ObjectInfo</code>, die zu speichernde Information
	 * @param list, ein <code>Vector</code>, die Liste, in die gespeichert werden soll
	 * @param maxListSize vom Typ <code>int</code>, begrenzt die Lnge dieser Liste
	 */
	private void insertInfoInList(ObjectInfo oi, Vector list, int maxListSize) {
		// fge in Liste ein
		insertInfoInList(oi,list);
		// begrenze die Lnge der Liste
		if(list.size() > maxListSize)
			list.setSize(maxListSize);
	}

	/**
	 * Methode zum Einfgen einer Information in eine Liste.<br>
	 * Beim Einfgen wird automatisch so sortiert, dass aktuellere Informationen
	 * in der Liste vorne stehen.
	 *
	 * @param or, eine <code>ObjectInfo</code>, die in einer Liste
	 *			gespeichert werden soll
	 * @param list, ein <code>Vector</code>, in den or gespeichert wird
	 */
	private void insertInfoInList(ObjectInfo oi, Vector list) {
		// suche ersten Eintrag der nicht aktueller ist als i
		int i = firstNotYounger(oi.time(), list);
		// gibt es einen solchen vor Ende der Liste
		if(i < list.size()) {
			// hat dieser den gleichen Zeitstempel wie das einzufgende Objekt
			if(oi.time()==((ObjectInfo)list.elementAt(i)).time()) {
				// lsche das gefundene Element
				list.removeElementAt(i);
			}
		}
		// fge die neue Information an dieser Stelle in die Liste ein
		list.insertElementAt(oi,i);
	}
	
	/**
	 * Methode zum Suchen einer <code>ObjectInfo</code> in einer nach
	 * Zeitwerten sortierten Liste, in der "jngere" Elemente vorn stehen.
	 * Es wird nach dem ersten Eintrag gesucht, der nicht jnger als ein
	 * bergebener Parameter ist.
	 *
	 * @param age vom Typ <code>int</code>, der Zeitwert.
	 * @param list vom Typ <code>Vector</code>, die Liste, in der nach
	 *		Eintrgen nicht jnger als der Zeitwert gesucht wird.
	 * @return eine <code>int</code>, der Index des gefundenen Elements bzw.
	 *		die Lnge der Liste bei erfolglosen Suchen.
	 */
	public int firstNotYounger(int age, Vector list) {
		int i = 0;
		// zhle Index i hoch, solange das Ende der Liste noch nicht erreicht ist
    	// und age kleiner ist als der Zeitwert der bislang gefundenen Objekte.
		while(i<list.size() && age < ((ObjectInfo)list.elementAt(i)).time()) {
			i++;
		}
		return i;
	}
	
/*-------------------------- Methoden zum "Merken" --------------------------*/	
	
	/**
	 * Methode zum Merken einer Information ber den Ball
	 *
	 * @param ballInfo vom Typ <code>ObjectInfo</code>, enthlt die 
	 * Information ber den Ball, die gespeichert werden soll
	 */
	public void keepBallInfo(ObjectInfo ballInfo) {
		// fge Ballinformation in Liste ein
		insertInfoInList(ballInfo, ballInfoList, ballInfoAmount);
	}
	
	/**
	 * Methode zum Merken einer Information ber den Spieler
	 *
	 * @param playerInfo vom Typ <code>ObjectInfo</code>, enthlt die 
	 * Information ber den Spieler, die gespeichert werden soll
	 */
	public void keepMyselfInfo(ObjectInfo myselfInfo) {
		// fge Spielerinformation in Liste ein
		insertInfoInList(myselfInfo, myselfInfoList, myselfInfoAmount);
		// fge auerdem in Liste mit Prognosen ein
		insertInfoInList(myselfInfo, myselfPredictionList, myselfPredictionAmount);
	}

	/**
	 * Methode zum Merken einer Aktion des Spielers
	 *
	 * @param action, ein <code>ActionBuffer</code>, die zu merkende Aktion
	 */
	public void keepAction(ActionBuffer action) {
		actionBufferList.addElement(action);
	}
	
	/**
	 * Methode zum Merken einer Drehung des Spielers
	 *
	 * @param oldInfo vom Typ <code>ObjectInfo</code>, die Informationen ber
	 * 		den Spieler vor der Drehung.
	 * @param turnTime vom Typ <code>int</code>, die Spielzeit,
	 *		zu der die Drehung ausgefhrt wurde.
	 * @param turnMoment vom Typ <code>double</code>, der Drehwinkel.
	 */
	public void keepTurn(ObjectInfo oldInfo, int turnTime, double turnMoment) {
		ObjectInfo turnInfo = new ObjectInfo(	turnTime+1,
									oldInfo.name(),
									oldInfo.distance(),
									StrategyLib.normAngle(oldInfo.direction()+turnMoment),
									oldInfo.position()	);
		insertInfoInList(turnInfo, myselfInfoList, myselfInfoAmount);
	}

	/**
	 * Methode zum Merken einer Bewegung des Spielers
	 *
	 * @param oldInfo vom Typ <code>ObjectInfo</code>, die Informationen ber
	 * 		den Spieler vor der Bewegung.
	 * @param dashTime vom Typ <code>int</code>, die Spielzeit,
	 *		zu der die Bewegung ausgefhrt wurde.
	 * @param dashPower vom Typ <code>double</code>, die Strke
	 *		der Beschleunigung des Spielers.
	 */
	public void keepDash(ObjectInfo oldInfo, int dashTime, double dashPower) {
		ObjectInfo dashInfo = new ObjectInfo(	dashTime+1,
									oldInfo.name(),
									oldInfo.distance(),
									oldInfo.direction(),
									StrategyLib.getPositionAfterMove(oldInfo.position(),dashPower/100,oldInfo.direction())
										);
		insertInfoInList(dashInfo, myselfPredictionList, myselfPredictionAmount);	
	}

/*------------------------- Methoden zum Erinnern ---------------------------*/	

	/**
	 * Gibt die neueste Prognose ber den Spieler zurck
	 * 
	 * @param playTime, eine <code>Integer</code>-Zahl, die Zeit, fr die die
	 * 		Prognose erstellt werden soll.
	 * @return eine <code>ObjectInfo</code>
	 * 		mit der neuesten Prognose ber den Spieler.
	 */
	 public ObjectInfo getMyselfPrediction(int playTime) {
	 	if(myselfInfoList.size()>0) {
	 		// hole die aktuellste Info ber den Spieler
	 		ObjectInfo me = (ObjectInfo)myselfInfoList.elementAt(0);
	 		// durchlaufe die Liste der gemerkten Aktionen
	 		for(Enumeration e = actionBufferList.elements(); e.hasMoreElements();) {
	 			// hole die nchste Aktion
	 			ActionBuffer action = (ActionBuffer)e.nextElement();
	 			// wenn diese zeitlich nach der letzten Info ber den Spieler liegt
	 			if(action.time() >= me.time()) {
	 				// behandle nur turns:
	 				if(action.type().equals("turn")) {
	 					// setzte Winkel, wie er nach der Drehung ist:
	 					me = new ObjectInfo( action.time()+1,me.name(),me.distance(),
									StrategyLib.normAngle(me.direction()+action.value()),
											 me.position() );
	 				}
	 				
	 				else if(action.type().equals("dash")) {
	 					me = new ObjectInfo( action.time()+1,me.name(),me.distance(),me.direction(),
									StrategyLib.getPositionAfterMove(me.position(),action.value()/100,me.direction())
											);
	 				}
	 			}
	 		}
	 		return me;
	 	}
	 	else {
	 		return null;
	 	}
	 }

	/**
	 * gibt alle gemerkten Infos ber den Spieler zurck
	 * 
	 * @return einen <code>Vector</code> von <code>ObjectInfo</code>s
	 * mit allen gemerkten Informationen ber den Spieler
	 */
	 public Vector getMyselfInfo() {
	 	return myselfInfoList;
	 }

	/**
	 * gibt alle gemerkten Infos ber den Ball zurck
	 * 
	 * @return einen <code>Vector</code> von <code>ObjectInfo</code>s
	 * mit allen gemerkten Informationen ber den Ball
	 */
	 public Vector getBallInfo() {
	 	return ballInfoList;
	 }
}
