/**
 * Diese Klasse wird zur Reprsentation wahrgenommener Objekte des Spieles
 * verwendet. Sie erweitert die Klasse orientiert sich voll an der Klasse
 * "ObjectPerception" des PI4-SimpleClient, erweitert diese aber, indem Sie
 * neben relativen auch absolte Koordinaten speichert.
 * Zu jedem Objekt werden die folgenden Informationen gespeichert:
 * <ul>
 * <li>der Zeitpunkt der Wahrnehmung</li>
 * <li>ein Name</li>
 * <li>die x-Koordinate</li>
 * <li>die y-Koordinate</li>
 * <li>der Abstand Spieler-Objekt</li>
 * <li>der Winkel, in dem der Spieler das Objekt sieht</li>
 * </ul>
 *
 * @see Memory
 * @author Gruppe 11 "DieElf"
 * @author <br>Ansprechpartner: <a href="mailto:rene@eDynamic.de">Ren Kubach</a>
 * @author <br>Ansprechpartner: <a href="mailto:bernius@web.de">Christian Bernius</a> 
 * @author <br>Ansprechpartner: <a href="mailto:brunnerrene@gmx.de">Ren Brunner</a> 
 * @version $Id: ObjectInfo.java,v 1.4 2002/07/07 20:56:55 mf Exp $
 */
 
public class ObjectInfo
{
    private int     time;
    private String  name;
    private double distance, direction;
    private double[] position = {Double.POSITIVE_INFINITY,Double.POSITIVE_INFINITY};
 
	/** 
	 * Konstruktor, erzeugt eine neue <code>ObjectInfo</code>
	 *
	 * @param time vom Typ <code>int</code>, Zeitpunkt der Wahrnehmung
	 * @param name vom Typ <code>String</code>, Namescode des Objectes wie
	 *		vom <code>rcssserver</code> bermittelt.
	 * @param distance vom Typ <code>double</code>, Abstand in dem das Objekt
	 *		wahrgenommen wurde.
	 * @param direction vom Typ <code>double</code>, Winkel, in dem das Objekt
	 *		wahrgenommen wurde.
	 * @param position vom Typ <code>double[]</code> der Form {x,y},
	 *		Absolutposition auf der das Objekt wahrgenommen wurde.
	 */
    public ObjectInfo(int time, String name, double distance, double direction, double[] position)
    {
        this.time=time;
      	this.name=name;
      	this.position = position;
      	this.distance = distance;
      	this.direction = direction;
    }

	/** 
	 * Konstruktor, erzeugt eine neue <code>ObjectInfo</code>, lt dabei
	 * aber den Eintrag <code>position</code> unangetastet.
	 *
	 * @param time vom Typ <code>int</code>, Zeitpunkt der Wahrnehmung
	 * @param name vom Typ <code>String</code>, Namescode des Objectes wie
	 *		vom <code>rcssserver</code> bermittelt.
	 * @param distance vom Typ <code>double</code>, Abstand in dem das Objekt
	 *		wahrgenommen wurde.
	 * @param direction vom Typ <code>double</code>, Winkel, in dem das Objekt
	 *		wahrgenommen wurde.
	 */
    public ObjectInfo(int time, String name, double distance, double direction)
    {
        this.time=time;
      	this.name=name;
      	this.distance = distance;
      	this.direction = direction;
    }

	/**
	 * Setzt die Absolutkoordinaten einer <code>ObjectInfo</code>
	 *
	 * @param position vom Typ <code>double[]</code> der Form {x,y},
	 *		Absolutposition auf der das Objekt wahrgenommen wurde.
	 */
	public void setPosition(double[] position) {
		this.position = position;
	}

	/**
	 * Setzt den Winkel, in dem ein Objekt wahrgenommen wurde.
	 *
	 * @param direction vom Typ <code>double</code>, der Winkel, in dem
	 *		das Objekt wahrgenommen wurde.
	 */
	public void setDirection(double direction) {
		this.direction = direction;
	}

    /** Zeitpunkt der Wahrnehmung */
    public int time()
    {
    	return time;
    }

    /** Name des Objektes */
    public String name()
    {
    	return name;
    }

	/**
	 * Position des Objektes
	 *
	 * @return ein <code>double</code>-Array der Form {x,y}
	 */
	 public double[] position()
	 {
	 	return position;
	 }

    /** x-Koordinate der Lage des Objektes */
    public double x()
    {
    	return position[0];
    }

    /** y-Koordinate der Lage des Objektes */
    public double y()
    {
    	return position[1];
    }

    /**
     * Abstand des Objektes zum Spieler
     */
    public double distance()
    {
    	return distance;
    }

    /**
     * Winkel zur Sichtlinie des Spielers
     */
    public double direction() {
    	return direction;
    }

}