import java.util.*;
/**
 * Diese Klasse reprsentiert einen einzelnen Spieler. Sie ist damit das
 * Herzstck des Clients. Hier werden Informationen angefordert, empfangene
 * Informationen verarbeitet und entsprechend darauf reagiert.
 *
 * @author Gruppe 11 "DieElf"
 * @author <br>Ansprechpartner: <a href="mailto:rene@eDynamic.de">Ren Kubach</a>
 * @author <br>Ansprechpartner: <a href="mailto:fooxbox@web.de">Markus Fuchs</a>
 * @author <br>Ansprechpartner: <a href="mailto:brunnerrene@gmx.de">Ren Brunner</a>
 * @version $Id: Player.java,v 1.15 2002/07/07 23:56:42 mf Exp $
 */
public class Player extends Thread
{

    /** Name des Teams */
    protected String teamName;
	/** interne Nummer des Spielers */
    protected int myNr;
    /** Seite, fr die der Spieler spielt */
    protected char mySide = ' ';

 	/** Spielzeit = aktueller Zyklus des <code>rcssserver</code>s */
 	private int playTime = -1;
    /** Weltzeit = Zeitstempel des letzten empfangenen <code>VisualEvents</code> */
    private int worldTime = -1;

    /** aktuelle Informationen ber den Spieler */
    private ObjectInfo me = null;
    /** aktuelle Informationen ber den Ball */
    private ObjectInfo theBall = null;
    /** Speicherzelle fr die lteste verwendbare Ballinfo */
    private ObjectInfo oldBall = null;

    /** Referenz zu Actions */
    protected Actions actions;
    /** Referenz zu EventControl */
    protected EventControl eventControl;
	/** Referenz zur StrategyLib */
	protected StrategyLib strategyLib;

    /** Referenz zum Weltmodell des Spielers */
    protected WorldModel worldModel;
    /** Boolean zum Anlegen eines Monitors  */
    protected boolean monitorActive;
	/** Referenz zum Gedchtnis des Spielers */
	protected Memory memory = new Memory();

    /** Konstante fr den Zustand "INIT" */
    final private int INIT	= 0;
    /** Konstante fr den Zustand "START" */
    final private int START	= 1;
    /** Konstante fr den Zustand "PLAY" */
    final private int PLAY	= 2;
    /** Konstante fr den Zustand "END" */
    final private int END	= 3;

    /** aktueller Status des Spielers,
     * kann "INIT", "START", "PLAY" oder "END" sein.
     * @see Player#INIT
     * @see Player#START
     * @see Player#PLAY
     * @see Player#END
     */
    private int status = INIT;

	public Player() {
	}

    /**
     * Konstruktor fr eine <code>Player</code>-Instanz.
     *
     * @param servername vom Typ <code>String</code>, gibt den Namen
     * des Servers an.
     * @param serverport vom Typ <code>int</code>, gibt den Port an,
     * an dem der Server eingehende Nachrichten empfngt.
     * @param teamName vom Typ <code>String</code>,
     * der Name des Teams, fr das der Spieler antritt.
     * @param myNr vom Typ <code>int</code>, gibt die Nummer des Spielers an;
     * ist zugleich die Nummer der playersArea, in der der Spieler spielen soll.
     * @param monitorActive, eine <code>boolean</code>, die besagt, ob die
     * Weltsicht des Spielers visualisert wird.
     */
    public Player(String servername, int serverport, String teamName, int myNr, boolean monitorActive)
    {
        this.teamName = teamName;
        this.myNr = myNr;
        ServerCommunication communication = new ServerCommunication(servername, serverport);
        actions = new Actions(communication);
        eventControl = new EventControl(this,communication);
        this.monitorActive = monitorActive; 
    }

/*------------------------- Informationsmanagement --------------------------*/

    /**
     * Diese Methode berschreibt die <code>run</code> Methode der
     * <code>JAVA</code>-Klasse <code>Thread</code>. Sie wird ausgefhrt,
     * wenn fr den <code>Player</code>-Thread mit <code>start()</code>
     * aufgerufen wurde.
     * Dieser Aufruf wird von <code>DieElf</code> gestartet.
     */
    public void run()
    {
		while(true) {
			switch (status) {
				// noch nicht angemeldet
				case INIT:
		        	// anmelden beim Server
        			init(teamName);
        			
        			
        /* alternative Startroutine
        *		case START:
        *			// Warten auf Spielfortsetzung bzw, zurcksetzen auf Startposition
		*			while( eventControl.receiveEvent() != 'b' ){
		*				System.out.println("Ich warte");
		*				actions.move(	PlayersArea.getStartPos(myNr)[0] ,
		*								PlayersArea.getStartPos(myNr)[1]	);
		*				}
        *			
        */			
				// vor Anpfiff und bei Spielunterbrechungen
				case START:
					for (int timeOut=0; timeOut<20; timeOut++){
						actions.move(	PlayersArea.getStartPos(myNr)[0] ,
										PlayersArea.getStartPos(myNr)[1]	);
					}				
					// warte auf ein BodyEvent
					if( eventControl.receiveEvent() == 'b' )
						// beobachte das Spiel
						watchTheGame();
				break;	
				// laufendes Spiel
				case PLAY:
					// warte auf ein BodyEvent
					if( eventControl.receiveEvent() == 'b' )
						// und spiele!
						play();
						
						// turn(10);
						
				break;

				// Spiel vorbei
				case END:
					// abmelden beim Server
					actions.bye();
				break;
			}
		}
    }


	/**
	 * ndert Seite, fr die der Spieler spielt.
	 *
	 * @param side vom Typ <code>char</code>, bezeichnet mit 'r' rechts
	 *		und mit 'l' links bezeichnet
	 */
	public void setMySide(char side) {
		mySide = side;
	}

	/**
	 * ndert die Nummer, die der Server dem Spieler gegeben hat
	 *
	 * @param myServerNr vom Typ <code>int</code>, die Nummer,
	 *		die der Server dem Spieler gegeben hat
	 */
	public void setMyServerNr(int myServerNr) {
	}
	
	/**
	 * ndert die Spieltzeit
	 *
	 * @param time vom Typ <code>int</code>, die neue Spielzeit
	 */
	public void setPlayTime(int time) {
		playTime = time;
	}
	
	/**
	 * ndert den Spielmodus "playMode"
	 *
	 * @param playMode, ein <code>String</code>, der den Spielmodus benennt
	 */
	public void setPlayMode(String playMode) {
		// vor dem Anspiel
		if(playMode.equals("before_kick_off")) {
			status = START;
		}
    	// Anspiel
    	else if(playMode.startsWith("kick_off")) {
    		// wir haben Anspiel
    		if(playMode.equals("kick_off_"+mySide))
    			status = PLAY;
    		// Gegner hat Anspiel
    		else
    			status = START;
    	}
    	// Freisto
    	else if(playMode.startsWith("free_kick")) {
    		// wir haben Freisto
    		if(playMode.equals("free_kick_"+mySide))
    			status = PLAY;
    		// Gegner hat Freisto
    		else
    			status = PLAY;
    	}
    	// Einwurf
    	else if(playMode.startsWith("kick_in")) {
    		// wir haben Einwurf
    		if(playMode.equals("kick_in_"+mySide))
    			status = PLAY;
    		// Gegner hat Einwurf
    		else
    			status = PLAY;
    	}
    	// Abschlag
    	else if(playMode.startsWith("goal_kick")) {
    		// wir haben Abschlag
    		if(playMode.equals("goal_kick_"+mySide))
    			status = PLAY;
    		// Gegner hat Abschlag
    		else
    			status = START;
    	}
    	// Eckball
    	else if(playMode.startsWith("corner_kick")) {
    		// wir haben Eckball
    		if(playMode.equals("corner_kick_"+mySide))
    			status = PLAY;
    		// Gegner hat Eckball
    		else
    			status = PLAY;
    	}
		// Spiel luft
		else if(playMode.equals("play_on")) {
			status = PLAY;
		}
    	// Tor gefallen
    	else if(playMode.startsWith("goal")) {
    		status = START;
    	}
    	// Halbzeit oder Beginn der Verlngerung
		else if(playMode.equals("half_time") || playMode.equals("time_extended")) {
			// ndere die Seite, fr die wir spielen (fr Weltmodell)
			// AUSKOMMENTIERT, DENN SERVER LSST DIE TEAMS NICHT DIE SEITEN
			// TAUSCHEN, SONDERN DREHT KOMPLETTES FELD (INKL. FLAGGEN)!
			//if(mySide=='r')
			//	mySide = 'l';
			//else
			//	mySide = 'r';
			status = START;
		}
		// Spiel vorbei
		else if( playMode.equals("time_over")) {
			status = END;
		}
	}

	/**
	 * Legt das Weltmodell des Spielers an
	 */
	public void createWorldModel() {
		worldModel= new WorldModel(teamName, mySide, monitorActive, myNr);
	}


	/**
	 * Aktualisiert das Weltmodell des Spielers.<br>
	 * Achtung: Dieser Vorgang luft in einem eigenen <code>Thread</code>,
	 * eventuelle nderungen am Quelltext, die den Bereich des
	 * Informationsmanagements, also insbesondere die Klassen
	 * <ul>
	 *   <li><code>Player</code></li>
	 *   <li><code>EventControl</code></li>
	 *   <li><code>WorldModel</code></li>
	 *   <li><code>Memory</code></li>
	 * </ul>
	 * bertreffen, bedrfen unbedingt der Synchronisation!
	 *
	 * @see WorldModel
	 * @param ein <code>String</code>, eine <code>see</code>-Nachricht
	 *		des Servers.
	 */
	public void updateWorldModel(String message) {
		// aktualisiere das Weltmodell
		worldModel.updateWorldModel(message);
		//aktualisiere dann (!!! - Multithreating - !!!) das Memory:
		updateMemory();		
	}

    /**
     * Aktualisiert das Gedchtnis des Spielers
     *
     * @see WorldModel
     * @see Memory
     */
	public void updateMemory() {
		// falls es Info ber mich gibt, merke diese!
		if(worldModel.getMyself()!=null)
			memory.keepMyselfInfo(worldModel.getMyself());
		// falls es Info ber den Ball gibt, merke diese!
		if(worldModel.getBall()!=null)
			memory.keepBallInfo(worldModel.getBall());
	}
	
/*------------------------- Entscheiden und Handeln -------------------------*/

	/**
	* Diese Methode veranlasst den Spieler zu Aktionen - sie baut aus den 
	* eintreffenden Informationen die idealen Reaktionen. Somit stellt sie die 
	* Zentrale des Entscheidungsfindungsprozesses dar.
	*/
    private void play()
    {
    	// hole neueste Prognose ber mich aus dem Memory
    	me = memory.getMyselfPrediction(playTime);

    	// hole neueste Info ber den Ball aus dem Memory
    	if(memory.getBallInfo().size() > 0)
			theBall = (ObjectInfo)memory.getBallInfo().elementAt(0);

		// wenn es NOCH keine Informationen ber mich gibt
		if(me==null) {
			// warte, tue nichts...	
		}
			
		// wenn die Informationen ber mich veraltet sind
		else if(Math.abs(playTime-me.time())>4) {
			// orientier dich neu:
			restoreInfo();
		}
			
		// wenn es aktuelle Informationen ber mich gibt
		else {

			// wenn ich keine aktuellen Infos ber den Ball hab
			if( !isBallActual() ) {
				// wenn ich nicht nahe an der RestPos bin
				if( !PlayersArea.isCloseToRestPos(me.position(), myNr) ) {
					// gehe dahin
					goTo(PlayersArea.getRestPos(myNr));
				}
				// wenn ich nahe an der RestPos bin
				else {
					// sehe dich nach Ball um
					findBall();
				}
			}

			// wenn ich Infos ber den Ball hab
			else {
				// wenn der Ball in meiner littleArea ist
				if( PlayersArea.isInLittleArea(theBall.position(), myNr) ) {
					// mach mir den Diego!
					diego();
				}
				// sonst:
				else {
					// wenn ich nicht nahe an der RestPos bin
					if( !PlayersArea.isCloseToRestPos(me.position(), myNr) ) {
						// gehe dahin
						goTo(PlayersArea.getRestPos(myNr));
					}
					// sonst
					else {
						// beobachte
						watchTheGame();
					}
				}
			}
		}
    }

	/**
	 * Prft, ob Informationen ber den Ball vorliegen,
	 * und, falls ja, ob diese aktuell sind.
	 *
	 * @return eine <code>boolean</code>, die besagt, ob aktuelle Ballinfos vorliegen
	 */
	private boolean isBallActual() {
		if(theBall==null) {
			return false;
		} else {
			if(Math.abs(playTime-theBall.time())>10)
				return false;
			else
				return true;
		}
	}

	/**
	 * Veranlasst den Spieler, sich nach neuem <code>Memory</code>-Input
	 * "umzusehen" (auch wrtl. zu verstehen!), um die aktuelle eigene Position
	 * bestimmen zu knnen.
	 */
	private void restoreInfo() {
		turn(35);
	}

	/**
	 * Veranlasst den Spieler, sich nach dem Ball umzusehen, um dessen aktuelle
	 * Position bestimmen zu knnen.
	 */
	private void findBall() {
		turn(35);
	}

	/**
	 * Veranlasst den Spieler, das Spiel zu beobachten, um rechtzeitig adequat
	 * auf Vernderungen der Spielsituation reagieren zu knnen.
	 */
	 
	 
	 // isBallActual!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!1
	 
	private void watchTheGame() {
		// wenn Du was vom Ball weit
		if(theBall != null) {
			if(Math.abs(theBall.direction()) > 20 )
				turn(theBall.direction() / 3);
		}
		// wenn Du nix vom Ball weit
		else {
			findBall();
		}
	}

	/**
	 * Bewegt den Spieler zu dem bergebenen Punkt
	 *
	 * @param aimPos vom Typ <code>double</code> gibt die Zielposition an
	 */	
	public void goTo(double[] aimPos) {
		// falls das Ziel nicht in Blickrichtung liegt, drehe zum Ziel
		if (Math.abs(StrategyLib.direction(me.position(), me.direction(), aimPos)) > 8) {
			turn(StrategyLib.direction(me.position(), me.direction(), aimPos));
		}
		// sonst gehe in Richtung des Ziels
		else {
			actions.dash(20 * StrategyLib.distance(me.position(), aimPos));
		}
	}

	/**
	 * Lt den Spieler spielen wie "Mr. Hand Gottes" himself.
	 */
	private void diego() {
		// wenn du noch zu weit weg bist vom Ball
		if(StrategyLib.distance(me.position(),theBall.position())>0.5)
			// gehe auf Abfangkurs
			goToBall();
		// wenn du nahe am Ball bist
		else
			// spiele den Ball
			playBall();
	} 

	/**
	 * Lt den Spieler den Ball spielen
	 */
	private void playBall() {
		actions.kick(100,StrategyLib.direction(me.position(),me.direction(),PlayersArea.getPassPos(myNr)));
	}


	/**
	 * Lt den Spieler zum Ball rennen
	 */
	private void goToBall() {
		
		// In den Koordinaten planPos soll der Ball abgefangen werden
		double[] planPos = new double[2];
		// future bestimm wieviele Spielzyklen vorausberechnet werden
		int future = (int) (0.5 * theBall.distance()); 
		// falls alte Ballinfo existieren
    	if(memory.getBallInfo().size() > 3)
    		// halte sie zum weitergeben bereit
			oldBall = (ObjectInfo)memory.getBallInfo().elementAt(3);
			// sonst benutze den aktuellsten Ball
		else 
			oldBall = theBall;
		// Berechne wann und wo du den Ball abfangen kannst
		planPos = StrategyLib.getFutureBallPosition(playTime, theBall, oldBall, future);
		// und geh hin
		goTo(planPos);
	}
	
	/**
	 * Bewegt den Spieler
	 *
	 * @see Actions
	 * @param power vom Typ <code>double</code>, die Kraft, mit der der Spieler
	 * 			beschleunigen soll.
	 */
	private void dash(double power) {
		// lasse den Spieler laufen
		actions.dash(power);
		// merke dir das!
		memory.keepDash(me,playTime,power);
	}
	
	/**
	 * Dreht den Spieler
	 *
	 * @see Actions
	 * @param angle vom Typ <code>double</code>, der Winkel, um den gedreht wird
	 */
	private void turn(double angle) {
		// drehe den Spieler
		actions.turn(angle);
		// merke dir das!
		memory.keepAction(new ActionBuffer(playTime,"turn",angle));
		// memory.keepTurn(me,playTime,angle);
	}
	
	/**
	 * Mldet den Spieler an
	 *
	 * @see Actions
	 * @param teamName vom Typ <code>String</code>, der Name des Teams
	 */
    public void init(String teamName){
    	actions.init(teamName);
	}

}