/**
 * Diese Klasse beschreibt die Spielposition eines Spielers
 *
 * @author Gruppe 11 "DieElf"
 * @author <br>Ansprechpartner: <a href="mailto:brunerrene@gmx.de">Ren Brunner</a>
 * @author <br>Ansprechpartner: <a href="mailto:rene@eDynamic.de">Ren Kubach</a>
 * @author <br>Ansprechpartner: <a href="mailto:fornarina@web.de">Diana Wachsmann</a>
 * @author <br>Ansprechpartner: <a href="mailto:fooxbox@web.de">Markus Fuchs</a> 
 * @author <br>Ansprechpartner: <a href="mailto:bernius@web.de">Christian Bernius</a> 
 * @version $Id: PlayersArea.java,v 1.8 2002/07/07 23:56:42 mf Exp $
 */
 
public class PlayersArea {
	
	/**
	 * Array fr die Positionen, zu denen die Spieler vor Anpfiff bewegt werden
	 */
	final private static double[][] startPos = {
		{ -50   , 0.1},
		{ -45   ,  -8},
		{ -42   ,  -3},
		{ -42   ,   3},
		{ -45   ,   8},
		{ -10   , -18},
		{ -20   , 0.1},
		{ -10   ,  18},
		{   0   , -15},
		{ -10   ,-0.1},
		{   0   ,  15}					  
	};

	/**
	* Im folgenden wird die Positionen definiert, auf denen die Player sich
	* im Ruhezustand erholen sollen
	*/
	final private static double[][] restPos = {
		{ -50   , 0.1},
		{ -45   ,  -8},
		{ -42   ,  -3},
		{ -42   ,   3},
		{ -45   ,   8},
		{ -10   , -18},
		{ -14   , 0.1},
		{ -10   ,  18},
		{  23   , -15},
		{  20   ,-0.1},
		{  23   ,  15}					  
	};

	/**
	* Im folgenden wird der Bereich definiert, in dem der Player auf den Ball
	* reagieren muss.<br>
	* Das Array enthlt x- und y-Koordinate des zentralen Punktes, halbe Lnge (x)
	* und halbe Breite (y) des Feldes.
	*/
	final private static double[][] littleArea = {
		{ -44.5  ,   0,  8   , 10 },
		{    -36 , -17, 17   , 18 },
		{    -42 ,  -5, 10.5 ,  7 },
		{    -42 ,   5, 10.5 ,  7 },
		{    -36 ,  17, 17   , 18 },
		{    0   , -17, 14   , 18},
		{    0   ,   0, 14   , 17 },
		{    0   ,  17, 14   , 18},
		{  31.25 , -17, 21.25, 17.5},
		{  31.25 ,   0, 21.25, 17 },
		{  31.25 ,  17, 21.25, 17.5},
	};
		
	/**
	* Im folgenden wird der Bereich definiert, in dem der Player sich aufhalten
	* darf.<br>
	* Das Array enthlt x- und y-Koordinate des zentralen Punktes, halbe Lnge (x)
	* und halbe Breite (y) des Feldes.
	*/
	final private static double[][] bigArea = {
		{ -44.5  ,   0,  8   , 10 },
		{ -26.25 , -17, 26.25, 18 },
		{ -26.25 ,  -8, 26.25, 17 },
		{ -26.25 ,   8, 26.25, 17 },
		{ -26.25 ,  17, 26.25, 18 },
		{    0   , -17, 20   , 18 },
		{    0   ,   0, 20   , 17 },
		{    0   ,  17, 20   , 18 },
		{  26.25 , -17, 26.25, 18 },
		{  26.25 ,   0, 26.25, 17 },
		{  26.25 ,  17, 26.25, 18 },
	};
	
	/**
	 * Array fr die Positionen, zu denen die Spieler passen sollen.
	 */
	final private static double[][] passPos = {
		{   0   ,   0 },
		{ -10   , -18 },
		{ -14   ,   0 },
		{ -14   ,   0 },
		{ -10   ,  18 },
		{  23   , -15 },
		{  23   ,   0 },
		{  23   ,  15 },
		{  52.5 ,   3 },
		{  52.5 ,   0 },
		{  52.5 ,  -3 },
	};

	/**
	 * gibt die Position zurck, zu der der Spieler vor Anpfiff bewegt wird
	 *
	 * @param playersAreaNr, eine <code>Integer</code>-Zahl, die die Nummer der
	 * PlayersArea angibt, in der der Spieler spielen soll
	 * @return ein <code>double[]</code>-Array, das die Position angibt, zu der
	 * der Spieler vor Anpfiff bewegt wird
	 */
	static public double[] getStartPos(int playersAreaNr) {
		return startPos[playersAreaNr];
	}

	/**
	 * gibt die Position zurck, an der der Spieler sich erholt
	 *
	 * @param playersAreaNr, eine <code>Integer</code>-Zahl, die die Nummer der
	 * PlayersArea angibt, in der der Spieler spielen soll
	 * @return ein <code>double[]</code>-Array, das die Position angibt, zu der
	 * der Spieler vor Anpfiff bewegt wird
	 */
	static public double[] getRestPos(int playersAreaNr) {
		return restPos[playersAreaNr];
	}
	
	/**
	 * gibt die Position zurck, auf die der Spieler passen soll
	 *
	 * @param playersAreaNr, eine <code>Integer</code>-Zahl, die die Nummer der
	 * PlayersArea angibt, in der der Spieler spielen soll
	 * @return ein <code>double[]</code>-Array, das die Position angibt, zu der
	 * der Spieler passen soll
	 */
	static public double[] getPassPos(int playersAreaNr) {
		return passPos[playersAreaNr];
	}

	
	/**
	 * Testet, ob eine Zahl zwischen zwei anderen liegt
	 *
	 * @param a, eine <code>double</code>, eine Intervallgrenze
	 * @param b, eine <code>double</code>, andere Intervallgrenze
	 * @param c, eine <code>double</code>, der Testpunkt
	 * @return eine <code>boolean</code>, die besagt, ob c zwischen a und b liegt
	 */
	static public boolean isBetween(double a, double b, double c) {
		if( (a<=c && c<=b) || (b<=c && c<=a) )
			return true;
		else
			return false;
	}
	
	/**
	 * Testet, ob eine bergebene Position innerhalb einer Area liegt
	 *
	 * @param area, ein <code>double[][]</code> die zu prfende Area
	 * @param pos vom Typ <code>double[]</code>, der zu prfende Punkt
	 * @param myNr, ein <code>int</code>, die Nummer des fragenden Spielers
	 * @return eine <code>boolean</code>, die besagt, ob die Position in
	 *         der Area ist.
	 */
	static private boolean isInArea(double[][] area, double[] pos, int myNr) {
		// gebe "ja" zurck, wenn x- und y- Koordinate von targetPos
		// beide zwischen den Grenzen der littleArea liegen
		if(	isBetween( area[myNr][0]-area[myNr][2], area[myNr][0]+area[myNr][2], pos[0]) &&
			isBetween( area[myNr][1]-area[myNr][3], area[myNr][1]+area[myNr][3], pos[1]) )
			return true;
		// sonst "nein"
		else
			return false;
	}

	/**
	 * Testet, ob eine bergebene Position innerhalb der littleArea liegt
	 *
	 * @param targetPos vom Typ <code>double[]</code>, der zu prfende Punkt
	 * @param myNr, ein <code>int</code>, die Nummer des fragenden Spielers
	 * @return eine <code>boolean</code>, die besagt, ob die Position in
	 *         der littleArea ist.
	 */
	static public boolean isInLittleArea(double[] targetPos, int myNr) {
		return isInArea(littleArea, targetPos, myNr);
	}

	/**
	 * Testet, ob eine bergebene Position innerhalb der bigArea liegt
	 *
	 * @param targetPos vom Typ <code>double[]</code>, der zu prfende Punkt
	 * @param myNr, ein <code>int</code>, die Nummer des fragenden Spielers
	 * @return eine <code>boolean</code>, die besagt, ob die Position in
	 *         der bigArea ist.
	 */
	static public boolean isInBigArea(double[] targetPos, int myNr) {
		return isInArea(bigArea, targetPos, myNr);
	}

	/**
	 * Testet, ob sich eine bergebene Position nahe am Restpoint befindet
	 *
	 * @param targetPos vom Typ <code>double[]</code>, der zu prfende Punkt
	 * @param myNr, ein <code>int</code>, die Nummer des fragenden Spielers
	 * @return eine <code>boolean</code>, die besagt, ob die Position nahe am
	 *         Restpoint befindet.
	 */
	static public boolean isCloseToRestPos(double[] targetPos, int myNr) {
		if(StrategyLib.distance(targetPos,getRestPos(myNr)) < 0.5)
			return true;
		else
			return false;
	}
	
	
// MAIN ZU TESTZWECKEN	
	static public void main(String[] args) {
		double[] a = {0,0};
		int nr = 2;
		if(isInLittleArea(a,nr))
			System.out.println("der naechste bidde");
		else
			System.out.println("neeee duuu...");
	}
}