import java.io.*;
import java.net.*;

/**
 * Die ServerCommunication ist fr das Senden und Empfangen der Kommunikations
 * Packets verantwortlich.
 *
 * @author Gruppe 11 "DieElf"
 * @author <br>Ansprechpartner: <a href="mailto:fooxbox@web.de">Markus Fuchs</a>
 * @author <br>Ansprechpartner: <a href="mailto:brunnerrene@gmx.de">Ren Brunner</a>
 * @version $Id: ServerCommunication.java,v 1.4 2002/07/07 20:56:55 mf Exp $
 */
 
public class ServerCommunication
{
	// Hier wird ein Socket vorbereitet, sozusagen der Netzwerkstecker
	private DatagramSocket socket;
	// Dann wird der Platz fr die IP-Adresse des Servers reserviert
	InetAddress serverIP;
	// Und fr den Port, damit man weiss an welchen "Eingang" Nachrichten geschickt werden sollen  
	public int port;

 	// Jetzt wird unsere Kommunikationsschnittstelle eingerichtet, 
 	// das geschieht am Anfang, wenn der Client erstmalig in Aktion tritt
	public ServerCommunication (String serverIP, int port)
	{
		try
		{   
			// eigener Socket
			this.socket = new DatagramSocket();
			// der Server bei dem wir uns anmelden
			this.serverIP = InetAddress.getByName(serverIP);
			// und der Initialport (meist 6000)
			this.port = port;
		}   
			// Hier werden Ausnahmefehler abgefangen
			catch (SocketException se)
			{	System.err.println("ServerCommunication: Kann keinen Socket erstellen!");
				System.exit(1);}
			catch (UnknownHostException uhe)
			{	System.err.println("ServerCommunication: Ich finde den Server nicht!");
				System.exit(1);}
	}
	
	
	// Diese Methode packt den String "action" in ein packet und schickt in an den Server
	public void send (String action)
	{
		try
		{
			// Konvertierung in den Typ Byte (nur der kann verschickt werden)
			byte [] buffer = action.getBytes();
			// Einrichten eines neuen leeren packets
			DatagramPacket packet = new DatagramPacket(buffer, buffer.length, serverIP, port);
			// Versand - Post lsst grssen!
			socket.send(packet);
		}
		// Hier werden Ausnahmefehler abgefangen
		catch (IOException ioe) {
			System.err.println("ServerCommunication: Packet ist auf halbem Weg beim Senden verreckt!");
		}
	}
	
	
	// Diese Methode sieht auf dem Socket nach, ob der Server etwas geschickt hat
	// Konvertiert den Inhalt in einen String und gibt ihn als Rckgabewert
	public String receive ()
	{
		// Ein Puffer der das was am Socket anliegt aufnimmt
		byte[] buffer = new byte[1024];
		// Und ein Packet das den Inhalt des Puffers zusammenrafft
		DatagramPacket packet = new DatagramPacket(buffer, buffer.length);
		
		// Hier wird der Socket aufgefordert ein neues Datenpacket vom Server entgegenzunehmen
		try 
		{	socket.receive(packet);}
		
		// Hier werden Ausnahmefehler abgefangen, falls der Server nichts geschickt hat
		// oder die Verbindung zusammenbricht
        catch (IOException ioe) {
        	System.err.println("ServerCommunication: Der Server antwortet nicht.");
        	System.exit(1);
        }
	
		// Jetzt schauen wir nochmal in das packet rein um zu sehen an welchem port der Server
		// ab sofort mit uns sprechen mchte, das merken wir uns gut!!!
        port = packet.getPort();
        // Zuletzt schicken wir das Empfangene als bereinigten String an die aufrufende Stelle
        return new String(packet.getData()).substring(0, packet.getLength()-1); 
	}
}
