/**
 * Diese Klasse enthlt grundstzliche Funktionen zum Spielen, Berechnen usw.
 *
 * @see <code>Player</code>
 * @author Gruppe 11 "DieElf"
 * @author <br>Ansprechpartner: <a href="mailto:rene@eDynamic.de">Ren Kubach</a>
 * @author <br>Ansprechpartner: <a href="mailto:fooxbox@web.de">Markus Fuchs</a>
 * @version $Id: StrategyLib.java,v 1.8 2002/07/07 20:56:55 mf Exp $
 */

public class StrategyLib
{
	/**
	 * Normiert einen bergebenen Winkel auf [-180,180]
	 *
	 * @param angle, eine <code>double</code>, der Winkel
	 * @return eine <code>double</code>, der normierte Winkel
	 */
	public static double normAngle(double angle) {
		double erg = angle;
		while(erg > 180)
			erg -= 360;
		while(erg < -180)
			erg += 360;
		return erg;
	}
	 
	/**
	 * Wendet Satz des Pythagoras an!
	 *
	 * @param a vom Typ double[2], die Katheten
	 * @return vom Typ double, die Hypotenuse
	 */
	public static double pythagoras(double[] a)
	{
		return pythagoras(a[0],a[1]);
	}

	/**
	 * Wendet Satz des Pythagoras an!
	 * 
	 * @param a vom Typ double, die eine Kathete
	 * @param b vom Typ double, die andere Kathete
	 * @return vom Typ double, die Hypotenuse
	 */
	
	public static double pythagoras(double a, double b) 
	{
		return Math.sqrt( Math.pow(a,2) + Math.pow(b,2) );
	}
	
	/**
	 * Gibt die Geschwindigkeit des Balles in x- und y-Richtung an,
	 * normiert auf einen Zyklus
	 *
	 * @param latestBallInfo vom Typ <code>ObjectInfo</code>,
	 * die aktuellsten Informationen ber den Ball
	 * @param prevBallInfo vom Typ <code>ObjectInfo</code>,
	 * die letzten Informationen ber den Ball
	 * @return ein <code>double</code>-Array, der die Geschwindigkeit
	 * des Balles im Metern pro Zyklus (=100ms), getrennt in x- und y-Richtung,
	 * angibt.
	 */
	public static double[] getBallSpeeds(ObjectInfo latestBallInfo, ObjectInfo prevBallInfo)
	{
		// berechne die Zeitdifferenz zwischen den Informationen
		int timespan = latestBallInfo.time() - prevBallInfo.time();
		double speed[] = {Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY};
		// berechne den vom Ball zurckgelegten Weg je in x- und y-Richtung
		// und teile ihn durch die dazu bentige Zeit, denn:
		// Geschwindigkeit = Weg pro Zeit ;-)
		for(int i=0;i<2;i++) {
			speed[i] = (latestBallInfo.position()[i] - prevBallInfo.position()[i]) / timespan;
		}
		return speed;
	}
	
	/**
	 * Bestimmt die aktuelle Geschwindigkeit des Balles aufgrund der aktuellen
	 * und der letzten Information ber den Ball
	 *
	 * @param latestBallInfo vom Typ <code>ObjectInfo</code>,
	 * die aktuellsten Informationen ber den Ball
	 * @param prevBallInfo vom Typ <code>ObjectInfo</code>,
	 * die letzten Informationen ber den Ball
	 * @ return ein <code>double</code>-Wert, der die (betragsmige) Geschwindigkeit
	 * des Balles im Metern pro Zyklus (=100ms) angibt.
	 */
	public static double getBallSpeed(ObjectInfo latestBallInfo, ObjectInfo prevBallInfo)
	{
		return pythagoras(getBallSpeeds(latestBallInfo,prevBallInfo));
	}
	
	/**
	 * Bestimmmt die zu erwartende naechste Position des Balles 
	 *
	 * @param actTime, die aktuelle Spielzeit als <code>Integer</code>-Zahl
	 * @param latestBallInfo, die letzte Information ber den Ball als
	 * <code>ObjectInfo</code>
	 * @param prevBallInfo, die vorletzte Information ber den Ball als
	 * <code>ObjectInfo</code>
	 * @param future, eine <code>Integer</code>-Zahl, die angibt, wie weit in
	 * die Zukunft gerechnet werden soll:
	 * <ul>
	 * <li>nchste Ballposition: future = 1</li>
	 * <li>bernchste Ballposition: future = 2</li>
	 * <li>usw.</li>
	 * </ul>
	 * @return ein <code>double</code>-Array der Form {x,y}, 
	 * die zu erwartende naechste Position des Balles 
	 */
	public static double[] getFutureBallPosition(	int actTime, 
													ObjectInfo latestBallInfo, 
													ObjectInfo prevBallInfo, 
													int future	)
	{	
		double[] futureBallPos = new double[2];
		
		// Zeitdifferenz zwischen den Memory-Informationen (in Zyklen)
		int memoTimeDiff = latestBallInfo.time() - prevBallInfo.time();
		// Zeitdifferenz zwischen der aktuellen Spielzeit und der letzten Memory-Information (in Zyklen)
		int actTimeDiff = actTime - latestBallInfo.time();
		
		double ballSpeed[] = getBallSpeeds(latestBallInfo, prevBallInfo);
		for(int i=0; i<2; i++) {
			futureBallPos[i] = latestBallInfo.position()[i] + (future + actTimeDiff)*ballSpeed[i]*Math.pow(0.9,future);
		}
		return futureBallPos;
	}

	/**
	 * Berechnet den Abstand zwischen zwei Punkten
	 *
	 * @param a vom Typ <code>double[]</code>, einer der Punkte
	 * @param b vom Typ <code>double[]</code>, einer der Punkte
	 * @return ein <code>double</code>-Wert, der Abstand zwischen a und b
	 */
	public static double distance(double[] a, double[] b) {
		double dist;
		dist = pythagoras(a[0]-b[0],a[1]-b[1]);
		return dist;
	} 

	/**
	 * Berechnet, wo sich ein Objekt nach einer Bewegung befindet
	 *
	 * @param pos, eine <code>double[]</code> der Form <code>{x,y}</code>,
	 *		beschreibt die Lage des Objektes vor der Bewegung
	 * @param distance vom Typ <code>double</code>, die Strecke, um die sich
	 *		das Objekt bewegt hat
	 * @param direction vom Typ <code>double</code>, die Richtung, in die sich
	 *		das Objekt bewegt hat
	 * @return eine <code>double[]</code> der Form <code>{x,y}</code>,
	 *		beschreibt die Lage des Objektes nach der Bewegung
	 */
	public static double[] getPositionAfterMove(double[] pos, double distance, double direction) {
		// Variable fr die Bewegung in x- und y-Richtung
		double[] move = {0,0};
		// Vorzeichen der Bewegungsrichtung
		boolean[] dirVz = new boolean[2];
		// Bewegung nach vorne / hinten
		dirVz[0] = (Math.abs(direction)>90)? true : false;
		// Bewegung nach rechts / links
		dirVz[1] = (direction<0)? true : false;
		
		// Sonderfall: keine Bewegung
		if(distance==0)	{
		}
		// Sonderfall: Bewegung nur in x-Richtung
		else if(direction==0 || Math.abs(direction)==180) {
			move[0] = distance;
		}
		// Sonderfall: Bewegung nur in y-Richtung
		else if(Math.abs(direction)==90) {
			move[1] = distance;
		}
		// "Normalfall"
		else {
			move[0] = Math.cos(Math.toRadians(direction)) * distance;
			move[1] = Math.sin(Math.toRadians(direction)) * distance;
		}
		// Addiere zur x- und y-Koordinate der Ausgangsposition
		// jeweils die x- und y-Koordinate der Bewegung
		for(int i=0; i<2; i++ ) {
			// beachte dabei das Vorzeichen der Bewegungsrichtung
			if(dirVz[i])
				// Bewegung nach vorn oder rechts
				pos[i] += move[i];
			else
				// Bewegungn nach hinten oder links
				pos[i] -= move[i];
		}
		
		return pos;
	}

	/**
	 * Gibt an, wo sich eine Position im Bezug auf den Spieler befindet
	 * 
	 * @param myPos vom Typ <code>double[]</code>, die Position des Spielers
	 * @param viewAngle vom Typ <code>double</code>, der Sichtwinkel des Spielers
	 * @param targetPos vom Typ <code>double[]</code>, die Position, deren relative
	 *        Koordinaten ermittelt werden sollen
	 * 
	 * @return eine <code>double</code>, der Winkel, in dem der Spieler targetPos
	 *			sieht.
	 */
	public static double direction(double[] myPos, double viewAngle, double[] targetPos)
	{
		// Ergebnisvariable 
		double krassemasse;
		
		// berechne die Katheten 
		double[] katheten = new double[2];
		for(int i=0; i<2; i++)
			katheten[i] = Math.abs(targetPos[i] - myPos[i]);		

		// berechne Winkel
		//Zielposition vor uns
		if (targetPos[0] > myPos[0]) {
			krassemasse =  Math.toDegrees(Math.atan(katheten[1] / katheten[0])); 
		}
		// auf einer Hoehe mit der Zielposition
		else if (targetPos[0] == myPos[0]) {
			krassemasse = 90;
				if(targetPos[1] == myPos[1])	
					krassemasse = 0;
		}
		// Zielposition hinter uns
		else {
			krassemasse = 90 + Math.toDegrees(Math.atan(katheten[0] / katheten[1]));
		}
		
		// bestimmt das Vorzeichen falls der Ball links von uns liegt
		if (targetPos[1] < myPos[1]) 
			krassemasse *= -1;

		// korrigiere Winkel um ViewAngle
		krassemasse -= viewAngle;
		// und normiere wieder auf [-180,180]
		krassemasse = normAngle(krassemasse);

		return krassemasse;
	}
	
// MAIN ZU TESTZWECKEN:	
	public static void main(String[] args) {
		double[] a = {0,0};
		double[] b = {10,-10};
		System.out.println(distance(a,b));
		System.out.println(direction(a,0,b));
	}
}