import java.util.*;
/**
 * Diese Klasse dient als Datenkontainer fr <code>VisualEvent</code>s,
 * die aber ins Datenformat <code>ObjectInfo</code>, also in
 * absolute x,y-Koordinaten, umgerechnet werden.
 *
 * @author Gruppe 11 "DieElf"
 * @author Ansprechpartner: <a href="mailto:rene@eDynamic.de">Ren Kubach</a>
 * @version 1.1
 */

public class WorldModel extends Thread
{
	/**
	 * Servernachricht,
	 * aus dessen Informationen das Weltmodell erstellt wird
	 */
	private String message;
	
	/** Name des Teams, fr das der Spieler antritt */
	private String teamName;
	/** Seite, fr die der Spieler spielt */
	private char mySide;
	/**
	 * Seite links vom Spieler, wenn dieser in Richtung des gegnerischen
	 * Tors sieht
	 */
	private char mySideL;
	/** Koordinaten des Spielers */
	private double[] myPosition  = { 	Double.POSITIVE_INFINITY ,
										Double.POSITIVE_INFINITY	};
	/**
	 * Sichtwinkel des Spielers.<br>
	 * Der Winkel ist normiert auf [-180,180]. Nulllinie ist die x-Achse
     * in Richtung des gegnerischen Tors. Der Wert ist positiv, wenn sich die
     * Sichtlinie rechts von der Nulllinie befindet, d.h. der Spieler muss sich
     * nach links drehen, um schnell das gegnerische Tor zu sehen.<br>
	 * Wert ist <code>POSITIVE_INFINITY</code>, wenn der Sichtwinkel nicht
	 * berechnet werden konnte oder noch nicht berechnet wurde.
	 */
	private double myViewAngle = Double.POSITIVE_INFINITY;

	/** Zeitstempel */
	private int time;
    /** Informationen ber den Spieler */
	private ObjectInfo myself;
	/** Informationen ber den Ball */
    private ObjectInfo ballInfo;
    /** Liste fr Informationen ber Fixpunkte (= Tore und Flaggen) */
	private Vector fixedPointList = new Vector();
	/** Liste fr Informationen ber Aussenlinien */
	private Vector lineList = new Vector();
	/** Liste fr Informationen ber Mitspieler */
	private Vector teamMemberList = new Vector();
	/** Liste fr Informationen ber gegnerische Spieler */
	private Vector oponentList = new Vector();
	
	/** Monitor zur Visualisierung des Weltbildes */
	ViewInit myMonitor = null;
	
/*----------------------------- Konstruktor(en) -----------------------------*/

	/**
	 * Konstruktor fr <code>WorldModel</code>
	 *
	 * @param teamName, ein <code>String</code>, der Name des Teams, fr das
	 * 			der Spieler spielt.
	 * @param side, ein <code>char</code>, die Seite fr die der Spieler kickt.
	 * @param monitorActive <code>boolean</code>, ob Spielermonitor aktiv.
	 * @param myNr <code>int</code>, (interne) Spielernummer in Monitor.
	 */
	public WorldModel(String teamName, char side, boolean monitorActive, int myNr) {
		// setzte teamName
		this.teamName = teamName;
    	// setze mySide und mySideL
		this.mySide = side;
    	if(side=='l')
    		// spielen wir links,
    		// so liegt die Seite "top" links vom Spieler, wenn er in Richtung
    		// des gegnerischen Tors sieht
    		this.mySideL = 't';
    	else
    		// spielen wir rechts
    		// ist dies die Seite "bottom"
    		this.mySideL = 'b';

		if(monitorActive) {
			myMonitor = new ViewInit(myNr);
			myMonitor.currCheckIfChanged = 1;
			myMonitor.prevCheckIfChanged = 1;
			System.out.println("Monitor wird instanziiert");
			myMonitor.start();
			
		}
	}

	/**
	 * Methode zum Aktualisieren des Weltmodells eines Spielers.<br>
	 * Speichert die bergebene Servernachricht in einer (Klassen-)globalen
	 * Variablen und ruft dann die <code>Thread</code>-Methode
	 * <code>start()</code> auf. Dadurch wird das weitere Vorgehen in einen
	 * eigenen <code>Thread</code> ausgelagert.
	 *
	 * @param message, die Servernachricht als <code>String</code>.
	 */
	public void updateWorldModel(String message) {
		// verarbeite nur Nachrichten, die lnger als 10 Zeichen sind
		// (andere sind unbrauchbar)
		if(message.length()>10) {
			this.message = message;
			// ohne eigenen Thread bis Synchronisation luft:
			// start();
			run();
		}
	}
	
	/** 
     * Diese Methode berschreibt die <code>run</code> Methode der
     * <code>JAVA</code>-Klasse <code>Thread</code>. Sie wird ausgefhrt,
     * wenn in der Methode <code>updateWorldModel()</code> die
     * <code>Thread</code>-Methode <code>start()</code> aufgerufen wird.
     * aufgerufen wurde.
	 */
    public void run() {
		// setzte Werte auf Anfangswerte zurck:
		resetAllValues();
		
		// parse Nachricht
		parseMessage(message);
		
		// bestimme die eigene Position
		setMyPosition();

		// falls die eigene Position bestimmt werden konnte
		if(myPosition[0]<Double.POSITIVE_INFINITY) {
			// speichere zu jedem Objekt die absolute Position,
			// errechnet anhand der eigenen:
			if(ballInfo!=null)
				ballInfo.setPosition(getOiCoordsByMine(ballInfo));
			
			for(Enumeration e = teamMemberList.elements(); e.hasMoreElements();) {
				ObjectInfo oi = (ObjectInfo)e.nextElement();
				oi.setPosition(getOiCoordsByMine(oi));
			}

			for(Enumeration e = oponentList.elements(); e.hasMoreElements();) {
				ObjectInfo oi = (ObjectInfo)e.nextElement();
				oi.setPosition(getOiCoordsByMine(oi));
			}
		}
		/** Sendet Informationen ueber den Ball, Spieler, Gegner das eigene Team und die Spielerseite an die Visualisierung*/
		if(myMonitor != null) {
			myMonitor.monitorUpdate(myself, ballInfo, teamMemberList, oponentList, mySide);
		}

		// Ausgaben zur Kontrolle:
		// System.out.println(time);
		// System.out.println("I am here: "+myPosition[0]+" / "+myPosition[1]+"............"+ ((ObjectInfo)fixedPointList.elementAt(0)).name() );
		// System.out.println("I can see "+fixedPointList.size()+" fixed points and "+lineList.size()+" line(s).");
    	//for(int i=0; i<2; i++)
    	//	if(fixedPointList.size()>i)
    	//		System.out.println("Next fixed point distance: "+((ObjectInfo)fixedPointList.elementAt(i)).distance()+
    	//						"\nNext fixed point name: "+((ObjectInfo)fixedPointList.elementAt(i)).name());
    	//if(ballInfo!=null)
		//	System.out.println("Ball found at "+ballInfo.x()+" / "+ballInfo.y());
		//else
		//	System.out.println("Ball not found.");
		//System.out.println("");
	}
		
/*---------------------------- interne Prozeduren ---------------------------*/

	/**
	 * Setzt alle Werte auf Anfangswerte zurck
	 * und leert ggf. gefllte Listen.<br>
	 *
	 * Dies ist notwendig, da
	 * <ul>
	 * <li>z.T. Sicherheitsabfragen gestartet werden, um feststellen zu knnen,
	 *		dass der bisherige Ablauf reibungslos vonstatten ging, und</li>
	 * <li>alte Informationen sonst in den Listen stehen bleiben wrden.</li>
	 * </ul>
	 */
	private void resetAllValues() {
		time = -1;
		myself = null;
		ballInfo = null;
		fixedPointList.clear();
		lineList.clear();
		teamMemberList.clear();
		oponentList.clear();
	}

	/**
	 * Parst aus der vom Server empfangenen Nachricht mglichst viele
	 * brauchbare Informationen.<br>
	 * Bricht, sollte eine unvollstndige Servernachricht empfangen worden
	 * sein, selbstndig an der Stelle ab, an der keine verwertbaren
	 * Informationen ber ein gesehenes Objekt mehr aus der Nachricht zu lesen
	 * sind.
	 *
	 * @param message vom Typ <code>String</code>, die vom Server empfangene
	 *			Nachricht.
	 */
	private void parseMessage(String message) {
    	// Zeiger: Ist Nachricht noch brauchbar?
    	// (bentigt, falls abgeschnittene Nachrichten ankommen)
    	boolean usable = true;
    	
        // StringTokenizer ist ein Utility, das einen String anhand
        // bestimmbarer Trennzeichen, hier: "(" und ")", in Teilstrings zerlegt
        // - hasMoreTokens() wird erfragt, ob es noch Teilstrings gibt
        // - nextToken() liefert den nchsten Teilstring
        //
        // zerlege message in Teilstrings:
        StringTokenizer tokenizer = null;
        tokenizer = new StringTokenizer(message, "()");

        // der erste Substring ist notwendigerweise von der Form
        //   "see " + zeitstempel +" "
        // - substring(i) liefert den Teilstring ab Stelle i (gezhlt wird 0,1,2,...)
        // - trim() entfernt Leerzeichen am Anfang und Ende
        //
        // der erste Teilstring enthlt den Zeitstempel:
        try {
        	time = Integer.parseInt(tokenizer.nextToken().substring(4).trim());
        // Fehler abfangen: kein Teilstring mehr gefunden
    	} catch (NoSuchElementException nsee) {
    		usable = false;    		
    	// Fehler abfangen: Teilstring nicht lnger als 3 Zeichen
    	} catch (StringIndexOutOfBoundsException sioobe) {
    		usable = false;    	
    	// Fehler abfangen: Unwandlung in eine Zahl nicht mglich
    	} catch (NumberFormatException nfe) {
    		usable = false;
    	}

        // notwendigerweise folgen jetzt immer
        //   - ein Teilstring mit dem Namen eines Objektes
        //   - ein Teilstring mit Lage-Informationen ber obiges Objekt
        //   - ein Teilstring der Form " " (falls danach noch was kommt)
        // und wieder von vorne
        //
        // diese werden jetzt abgearbeitet:
        while (usable && tokenizer.hasMoreTokens())
        {
			// im Folgenden versuchen wir immer wieder, folgendes abzuarbeiten:
			//   - entweder einen Teilstring,
			// 	   wenn der von dieser von der Form " " ist
			//	   und tun in diesem Fall nichts
			//   - oder zwei Teilstrings ab,
			//     nmlich Namen und Lage-Informationen
			//     und legen aus diesen Daten eine ObjectInfo an
			
			try {
				// zuerst der Teilstring mit dem Namen des Objektes
				// nameCode ist der Name eines Objektes, so wie der rcssserver es
				// benennt, z.B. "f t r 20"
				String nameCode = tokenizer.nextToken();

	          	// zunchst fangen wir Teilstrings der Form " " ab,
    	      	if (!nameCode.equals(" "))
        	  	{
	    			// der nchste Teilstring enthlt Informationen
    				// ber die Lage des Objektes namens nameCode       
        	   		String locationData = tokenizer.nextToken();
					// diesen zerlegen wir jetzt wie oben message,
					// nur ist nun das Leerzeichen das Trennzeichen:
					StringTokenizer locationDataTokenizer = new StringTokenizer(locationData, " ");
					
					// der erste Teilstring enthlt Infos ber Entfernung
					double distance = Double.valueOf(locationDataTokenizer.nextToken()).doubleValue();
					// der zweite Teilstring enthlt Infos ber Richtung
					double direction = Double.valueOf(locationDataTokenizer.nextToken()).doubleValue();
					// jetzt sind alle Daten verfgbar, um sie in einer ObjectInfo
					// zu speichern und diese ans VisualEvent zu hngen
					ObjectInfo objectInfo = new ObjectInfo(time,nameCode,distance,direction);
					// speichere diese in der entsprechenden Liste
					distributeToLists(objectInfo);
				}
				
			// Fehler abfangen: einer der Teilstrings ist nicht angekommen	
			} catch (NoSuchElementException nsee) {
    				usable = false;
   			// Fehler abfangen: ein Teilstring hat kein Zahlenformatm, obwohl
   			// er dies eigentlich haben mte (z.B. direction: "12.")
   			} catch (NumberFormatException nfe) {
					usable = false;
   			}
        }		
	}

	/**
	 * Methode zum Verteilen der <code>ObjectInfo</code>s eines
	 * <code>Event</code>s auf die entsprechenden internen Listen. Setzt
	 * auerdem die Zeit des Weltmodells.
	 *
     * @param event, ein <code>VisualEvent</code>, enthlt
     * die <code>ObjectInfo</code>s, die verteilt werden sollen.
     * @param teamName, ein <code>String</code>, der Name des Teams, fr das
     * der Spieler spielt
	 */
	private void distributeToLists(ObjectInfo oi) {
		// verteile die ObjectInfo anhand des Anfangsbuchstabens
		// auf die richtige Liste            
		switch (oi.name().charAt(0)) {
			// Informationen ber den Ball
			case 'b':
				ballInfo = oi;
			break;
			// Informationen ber Spieler
			case 'p':
				// Informationen ber Mitspieler
				// Achtung: Teamnamen werden vom Server immer mit
				// hochgestellten Anfhrungs- und Endzeichen (") versehen!
				if(oi.name().startsWith("p \""+teamName+"\""))
					insertInList(oi, teamMemberList);
				// Informationen ber Gegenspieler
				else
					insertInList(oi, oponentList);
			break;
			// Informationen ber Tore
			case 'g':
			// Informationen ber Flagge
			case 'f':
				insertInList(oi, fixedPointList);
			break;
			// Informationen ber Linien
			case 'l':
			    insertInList(oi, lineList);
			break;
		}
	}

   /**
    * Methode zum sortierten Einfgen einer <code>ObjectInfo</code>
    * in eine der internen Liste fr <code>ObjectInfo</code>s.<br>
    * Es wird nach Entfernung sortiert, wobei die nahen Objekte in der Liste
    * vorne sind. Arbeitet  la InsertionSort, geht von leeren oder bereits
    * sortierten Listen aus.
    * 
    * @param oi vom Typ <code>ObjectInfo</code>,
    * das Element, das eingefgt werden soll
    * @param list vom Typ <code>Vector</code>,
    * die Liste, in die eingefgt werden soll
    */
    private void insertInList(ObjectInfo oi, Vector list) {
    	int i=0;
    	// zhle Index hoch, solange das Ende der Liste noch nicht erreicht ist
    	// und der Abstand zum aktuellen Objekt grsser ist als zu den bisherigen der Liste
		while(i<list.size() && oi.distance()>((ObjectInfo)list.elementAt(i)).distance()) {
			i++;
		}
		list.insertElementAt(oi,i);
	}

	/**
	* Methode zur Bestimmung der Absolut-Koordinaten eines Fixpunktes,
	* d.h. einer Flagge oder eines Tores, nach rccsserver-Version 7.0
	*
	* @param oiName, ein <code>String</code>, der den NameCode der Flagge
	* oder des Tores beinhaltet
	* @return <code>double[]</code>-Array der Form {x,y}
	* @see <code>ObjectInfo</code>
	*/
	private double[] getFixedPointCoords(String oiName) {
		// verzichte auf diese Info, wenn Fixpunkt eine Flagge (und kein Tor)
		// ist, d.h. verwerfe den ersten Teil ("f ") der Namenskodierung
		if(oiName.startsWith("f "))
			oiName = oiName.substring(2);
			
		double x=0.0, y=0.0;
		
		// zerlege Namenskodierung in drei Buchstaben c1, c2 und c3
		StringTokenizer t = new StringTokenizer(oiName," ");
		// erster Buchstabe
		char c1 = t.nextToken().charAt(0); 
		// zweiter Buchstabe, falls noch vorhanden, sonst Leerzeichen
		char c2 = (t.hasMoreTokens()) ? t.nextToken().charAt(0) : ' ';
		// dritter Buchstabe, falls noch vorhanden, sonst Leerzeichen
		char c3 = (t.hasMoreTokens()) ? t.nextToken().charAt(0) : ' ';
		
		// Flaggen am Tor
		if(c1=='g') {
			x=52.5;
			if(c3!=' ') y=7.01;
			if(c2==mySide) x*=-1;
			if(c3==mySideL) y*=-1;
		}
		// Flaggen am Strafraumrand
		else if(c1=='p') {
			x=36.5;						// 52.5 - Strafraumlnge, noch nachschlagen!
			if(c3!='c') y=20.0;			// halbe Strafraumbreite, noch nachschlagen!
			if(c2==mySide) x*=-1;
			if(c3==mySideL) y*=-1;
		}		
		// Flaggen ausserhalb des Feldes		
		else if(c3!=' ' || c2=='0') {
			if(c1=='r' || c1=='l') {
				x=57.5;
				if(c3!=' ') y=10*Double.valueOf(String.valueOf(c3).toString()).doubleValue();
				if(c1==mySide) x*=-1;
				if(c2==mySideL) y*=-1;
			}
			else if(c1=='b' || c1=='t') {
				y=39.0;
				if(c3!=' ') x=10*Double.valueOf(String.valueOf(c3).toString()).doubleValue();
				if(c1==mySideL) y*=-1;
				if(c2==mySide) x*=-1;
			}
		}
		// Flaggen am Spielfeldrand
		else if(c2=='b' || c2=='t') {
			y=34.0;
			if(c1!='c') x=52.5;
			if(c1==mySide) x*=-1;
			if(c2==mySideL) y*=-1;
		}
		double erg[] = {x,y};
		return erg;
	}
	
    /**
     * Methode zur Bestimmung der eigenen Position,
     * setzt die internen Variablen myViewAngle (wenn mglich) und myPosition.
     */
	private void setMyPosition() {
		// wenn eine Linie gesehen wurde
		if(lineList.size()>0) {
			// ermittle und setze myViewAngle
			setMyViewAngle();
			// wenn auerdem eine Flagge gesehen wurde
			if(!fixedPointList.isEmpty()) {
				// ermittle anhand derer die eigene Position

	    		ObjectInfo closestFixedPoint = (ObjectInfo)fixedPointList.elementAt(0);
		    	myPosition = getMyCoordsByFP(closestFixedPoint);

			}
			myself = new ObjectInfo(time,"myself",0,myViewAngle,myPosition);
		}

    	// es fehlt: Fehlerbehandlung:
    	// Was, wenn der Spieler das Feld verlassen hat?
    	// Was, wenn keine Linie gesehen wird?
    	// Was, wenn nur eine Flagge gesehen wird?
    	
    	// Lsung knnte eine Prozedur sein, die dem Spieler die Ermittlung der
    	// eigenen Position lediglich mittels zweier Flaggen ermglicht,
    	// also ohne myViewAngle. Dann knnte sich der Spieler im 5m-Rand um das
    	// Spielfeld noch orientieren und rechtzeitig umdrehen.    	
	}		     

	/**
	 * Methode zur Bestimmung des Sichtwinkels des Spielers,
	 * setzt die interne Variable myViewAngle.
	 */
	private void setMyViewAngle() {
   		// whle zunchst die erste gesehene Linie als die nchste
   		ObjectInfo helischLine = (ObjectInfo)lineList.elementAt( lineList.size()-1 );
   		// bestimme nun den vom Server gemessenen Winkel zwischen
   		// Sichtlinie des Spielers und der x-Achse in positiver Richtung
		double measuredAngle = helischLine.direction();

		// Linien oben und unten
		if(helischLine.name().charAt(2)=='t' || helischLine.name().charAt(2)=='b') {
			if(helischLine.name().charAt(2)==mySideL) {
				if(measuredAngle<0)
					myViewAngle = -180-measuredAngle;
				else 
					myViewAngle = 0-measuredAngle;
			} else {
				if(measuredAngle<0)
					myViewAngle = 0-measuredAngle;
				else
					myViewAngle = 180-measuredAngle;
			}
		}
		// Linien rechts und links
		else {
			if(helischLine.name().charAt(2)==mySide) {
				if(measuredAngle<0)
					myViewAngle =  90-measuredAngle;
				else
					myViewAngle = -90-measuredAngle;				
			} else {
				if(measuredAngle<0) 
					myViewAngle = -90-measuredAngle;
				else
					myViewAngle =  90-measuredAngle;
			}
		}
	}

	/**
	* Methode zur Bestimmung der eigenen Position anhand eines Fixpunktes,
	* unter der Voraussetzung, dass der Sichtwinkel <code>myViewAngle</code>
	* des Spielers bekannt ist.
	*
	* @param fp vom Typ <code>ObjectInfo</code>
	* @see <code>ObjectInfo</code>
	* @return <code>double[]</code>-Array der Form {x,y}
	*/
	private double[] getMyCoordsByFP(ObjectInfo fp) {
		double myXY[] = new double[2];
		double fpXY[] = getFixedPointCoords(fp.name());
		myXY[0] = fpXY[0]-fp.distance()*Math.cos(Math.toRadians(fp.direction()+myViewAngle));
		myXY[1] = fpXY[1]-fp.distance()*Math.sin(Math.toRadians(fp.direction()+myViewAngle));
		return myXY;
	}

	/**
	* Methode zur Bestimmung der Absolut-Koordinaten einer
	* <code>ObjectInfo</code> anhand der eigenen - deren Kenntnis wird
	* ebenso vorausgesetzt wie die der Sichtlinie <code>myViewAngle</code>
	* des Spielers.
	*
	* @param oi vom Typ <code>ObjectInfo</code>
	* @see <code>ObjectInfo</code>
	* @return <code>double[]</code>-Array der Form {x,y}
	*/
	private double[] getOiCoordsByMine(ObjectInfo oi) {
		double oiXY[] = new double[2];
		oiXY[0] = myPosition[0] + oi.distance() * Math.cos(Math.toRadians(oi.direction()+myViewAngle));
		oiXY[1] = myPosition[1] + oi.distance() * Math.sin(Math.toRadians(oi.direction()+myViewAngle));
		return oiXY;
	}
	

/*---------------------------- public Prozeduren ----------------------------*/

    /**
     * gibt des Ball als <code>ObjectInfo</code> zurck
     *
     * @see ObjectInfo
     * @return eine <code>ObjectInfo</code> mit Infos ber den Ball
     */
    public ObjectInfo getBall() {
     	return ballInfo;
  	}
    
    /**
     * gibt den Spieler als <code>ObjectInfo</code> zurck
     *
     * @see ObjectInfo
     * @return eine <code>ObjectInfo</code> mit Infos ber den Spieler,
     * dabei ist <code>direction</code> <code>myViewAngle</code>
     */
    public ObjectInfo getMyself() {
     	return myself;
  	}
    
	/**
	 * gibt den Zeitstempel des Weltmodells zurck
	 *
	 * @return eine <code>Integer</code>-Zahl, die die Spielzeit
	 * angibt, auf die sich dieses Weltmodell bezieht.<br>
	 * gibt <code>-1</code> zurck, wenn das <code>Event</code> leer war,
	 * d.h. keine Informationen enthielt.
	 */
	public int getTime() {
	 	return time;
	}

	 /**
	 * gibt eine Liste der gesehenen Mitspieler zurck
	 *
	 * @see ObjectInfo
	 * @return einen <code>Vector</code>, in dem Informationen ber
	 * die Mitspieler als <code>ObjectInfo</code>s gespeichert sind
	 */
	 public Vector getTeamMemberList() {
	 	return teamMemberList;
	 }

	 /**
	 * gibt eine Liste der gesehenen Gegenspieler zurck
	 *
	 * @see ObjectInfo
	 * @return einen <code>Vector</code>, in dem Informationen ber
	 * die Gegenspieler als <code>ObjectInfo</code>s gespeichert sind
	 */
	 public Vector getOponentList() {
	 	return oponentList;
	 }

}